<?php
/*
Plugin Name: Simple Neon Login
Plugin URI: 
Description:  Authenticate WordPress Users on Neon.
Version: 1.1.0
Author: Tony Hanson, Webmaster: Dallas Genealogical Society
Author URI: http://dallasgenealogy.org

17 January 2016
Version 1.1.0 addresses problems caused by case insensitivity when deleting users
in the authenticate function.
*/

class SimpleNeonLogin {
	static $instance = false;
	var $prefix      = 'sll_';
	var $settings    = array();
	
	public function __construct () {
		$this->settings = $this->get_settings_obj( $this->prefix );

		add_action('admin_init', array($this, 'save_settings') );
		add_action('admin_menu', array($this, 'menu') );

		if ( str_true($this->get_setting('enabled')) ) {
			add_filter('authenticate', array($this, 'authenticate'), 1, 3);
		}

		register_activation_hook( __FILE__, array($this, 'activate') );

		// If version is false, and old version detected, run activation
		if( $this->get_setting('version') === false || get_option('simpleldap_domain_controllers', false) !== false ) $this->activate();
		
		//
		// Check if cURL is enabled on the server
		//
		if ( ! function_exists('curl_version') ) {
			echo '<p style="color: red">The cURL extension is disabled on your server. This plugin requires cURL to be enabled.</p>';
		}
	}

	public static function getInstance () {
		if ( !self::$instance ) {
		  self::$instance = new self;
		}
		return self::$instance;
	}

	function activate () {
		// Default settings

		$this->add_setting('enabled',       "false");
		
		$this->add_setting('role',          "Subscriber");
		$this->add_setting('org_id',        "false");
		$this->add_setting('api_key',       "false");
		$this->add_setting('exempt_ids',    "false");
		$this->add_setting('first_name',    "false");
		$this->add_setting('last_name',     "false");
		$this->add_setting('email_address', "false");

		if( $this->get_setting('version') === false ) {
			$this->set_setting('enabled', 'true');
 		}
	}

	function menu () {
		add_options_page("Simple Neon Login", "Simple Neon Login", 'manage_options', "simple-neon-login", array($this, 'admin_page') );
	}

	function admin_page () {
		include 'Simple-Neon-Login-Admin.php';
	}

	function get_settings_obj () {
		return get_option("{$this->prefix}settings", false);
	}

	function set_settings_obj ( $newobj ) {
		return update_option("{$this->prefix}settings", $newobj);
	}

	function set_setting ( $option = false, $newvalue ) {
		if( $option === false ) return false;

		$this->settings = $this->get_settings_obj($this->prefix);
		$this->settings[$option] = $newvalue;
		return $this->set_settings_obj($this->settings);
	}

	function get_setting ( $option = false ) {
		if($option === false || ! isset($this->settings[$option]) ) return false;

		return apply_filters($this->prefix . 'get_setting', $this->settings[$option], $option);
	}

	function add_setting ( $option = false, $newvalue ) {
		if($option === false ) return false;

		if ( ! isset($this->settings[$option]) ) {
			return $this->set_setting($option, $newvalue);
		} else return false;
	}

	function get_field_name($setting, $type = 'string') {
		return "{$this->prefix}setting[$setting][$type]";
	}

	function save_settings()
	{
		if( isset($_REQUEST["{$this->prefix}setting"]) && check_admin_referer('save_sll_settings','save_the_sll') ) {
			$new_settings = $_REQUEST["{$this->prefix}setting"];

			foreach( $new_settings as $setting_name => $setting_value  ) {
				foreach( $setting_value as $type => $value ) {
					if( $type == "array" ) {
						$this->set_setting($setting_name, explode(";", $value));
					} else {
						$this->set_setting($setting_name, $value);
					}
				}
			}

			add_action('admin_notices', array($this, 'saved_admin_notice') );
		}
	}

	function saved_admin_notice(){
	    echo '<div class="updated">
	       <p>Simple Neon Login settings have been saved.</p>
	    </div>';

	    if( ! str_true($this->get_setting('enabled')) ) {
			echo '<div class="error">
				<p>Simple Neon Login is disabled.</p>
			</div>';
	    }
	}

	function authenticate ($user, $username, $password) {
                //DEBUG
		//echo "authenticate function called</p>";
		
		//
		// Required for the delete_wp_user command
		//
		include './wp-admin/includes/user.php';
		
		//DEBUG
		//echo "<p>authenticate function has been called<p/>";
		
		// If previous authentication succeeded, respect that
		if ( is_a($user, 'WP_User') ) { return $user; }
		
		//DEBUG
		//echo "<p>authenticate: User is NOT already logged in<p/>";
		
		// Determine if user a local admin
		$local_admin = false;
		$user_obj = get_user_by('login', $username); 
		if( user_can($user_obj, 'update_core') ) $local_admin = true;
		
		//DEBUG
		//echo "<p>local_admin value: [$local_admin] <p/>";
		
		if ( empty($username) || empty($password) ) {
			$error = new WP_Error();

			if ( empty($username) )
				$error->add('empty_username', __('<strong>ERROR</strong>: The username field is empty.'));

			if ( empty($password) )
				$error->add('empty_password', __('<strong>ERROR</strong>: The password field is empty.'));

			return $error;
		}

		//DEBUG
		//echo "<p>Calling neon_auth function...<p/> ";
		
		// Sweet, let's try to authenticate our user and pass against Neon
		$auth_result = $this->neon_auth($username, $password );
		
		//DEBUG
		//echo "<p>neon_auth function complete: Checking results. auth_result value: [$auth_result]<p/>";

		if( $auth_result ) {
			// Authenticated
			$user = get_user_by('login', $username);
			
			//DEBUG
			//echo "<p>username: [$username] was validated by Neon</p>";

			if ( ! $user || ( strtolower($user->user_login) !== strtolower($username) ) )  {
				//DEBUG
				//echo "<p>New User</p>";

				//
				// Get the current user information
				//
				$user_info = $this->get_user_data( $username );
				
				//
				// Add Name and Email Address ( defined in neon_auth() )
				//
				$user_info[first_name] = $this->get_setting('first_name');
				$user_info[last_name]  = $this->get_setting('last_name');
				$user_info[user_email] = $this->get_setting('email_address');			
								
				$new_user = wp_insert_user( $user_info );
				

				if( ! is_wp_error($new_user) )
				{
					//DEBUG
		                        //echo "<p>Successful Login<p/>";
					
					// Successful Login
					$new_user = new WP_User($new_user);
					do_action_ref_array($this->prefix . 'auth_success', array($new_user) );

					return $new_user;
				}
				else
				{
					//DEBUG
		                        //echo "<p>Neon credentials are correct and user creation is allowed but an error occurred creating the user in WordPress<p/>";
					
					do_action( 'wp_login_failed', $username );
					return new WP_Error("{$this->prefix}login_error", __('<strong>Simple Neon Login Error</strong>: Neon credentials are correct and user creation is allowed but an error occurred creating the user in WordPress. Actual error: '.$new_user->get_error_message() ));
				}
			} else {
				//DEBUG
		                //echo "<p>Existing User - UserID: [$user->ID]<p/>";
				
				return new WP_User($user->ID);
			}
		} else {
		   //
		   // User was not validated by Neon
		   // If the user exists in the WP Users table it should be deleted
		   //
		   
		   //DEBUG
		   //echo "<p>User was not authenticated by Neon!<p/>";
		   
		   //
		   // Create an array of exempt Usernames (defined on the settings page)
		   //
		   $exempt = explode(";", $this->get_setting('exempt_ids'));
		   
		   //
		   // Delete the user unless they are an admin or were specified in the
		   // Locally Verified Usernames field on the Settings page
		   //
		   // Modified 17 January 2016 to address a case sensitivity problem
		   //
		   //if ( !$local_admin && !in_array($username, $exempt) ) {
		   
		   if ( !$local_admin && !in_array(strtolower($username), array_map('strtolower', $exempt)) ) {
		      
		      $user = get_userdatabylogin($username);
                      //echo $user->ID; // prints the id of the user
		   
		      if ($user)
		      {
		   	   //
			   // We information about the user, should have the UserID.
			   // Delete this user from the Users table
			   //
			
			   //echo "<p>I am going to delete this ID - local_admin: [$local_admin]</p>";
			   wp_delete_user( $user->ID );
		      }
		   }
		}

		do_action($this->prefix . 'auth_failure');
		return false;
	}

	function get_user_data( $username ) {
		$user_data = array(
			'user_pass' => md5( microtime() ),
			'user_login' => $username,
			'user_nicename' => '',
			'user_email' => '',
			'display_name' => '',
			'first_name' => '',
			'last_name' => '',
			'role' => $this->get_setting('role')
		);

		if( is_array($userinfo) ) {
			$user_data['user_nicename'] = $userinfo['givenname'][0] . ' ' . $userinfo['sn'][0];
			$user_data['user_email'] 	= $userinfo['mail'][0];
			$user_data['display_name']	= $user_data['user_nicename'];
			$user_data['first_name']	= $userinfo['givenname'][0];
			$user_data['last_name'] 	= $userinfo['sn'][0];
		}

		return apply_filters($this->prefix . 'user_data', $user_data);
	}
	
	//#################################################################################################
	//
        // Functions from Neon.
        // See http://help.neoncrm.com/api
        //
        // The login request authenticates your API Key and Organization ID. 
        // You will receive a session ID from NeonCRM, which will allow you to 
        // make further API requests. This request requires Organization ID and API key 
        // as parameters. The Session ID is saved as a session variable. This is 
        // automatically injected into further requests.
        //
	function login($apiKey, $orgId)
        {
           //DEBUG
	   //echo "<p>login function called<p/>";
	   
	   $ch = curl_init();
           $url = 'https://api.neoncrm.com/neonws/services/api/common/login';
           $params = 'login.apiKey=' . $apiKey . '&login.orgid=' . $orgId;
           curl_setopt($ch,CURLOPT_URL, $url);
           curl_setopt($ch,CURLOPT_POST, TRUE);
           curl_setopt($ch,CURLOPT_POSTFIELDS, $params);
           curl_setopt($ch,CURLOPT_RETURNTRANSFER, TRUE);
           $result = curl_exec($ch);
           curl_close($ch);
           $response = json_decode($result);
	   return $response;
        }
	
	//
        // RetrieveIndividualAccount
        //
	function RetrieveIndividualAccount($sessionId, $accountId)
        {
           //echo "<br/>RetrieveIndividualAccount function called<br/>";
         
           $ch     = curl_init();
           $url    = 'https://api.neoncrm.com/neonws/services/api/account/retrieveIndividualAccount'; 
           $params = 'userSessionId=' . $sessionId . '&accountId=' . $accountId;
         
           curl_setopt($ch,CURLOPT_URL, $url);
           curl_setopt($ch,CURLOPT_POST, TRUE);
           curl_setopt($ch,CURLOPT_POSTFIELDS, $params);
           curl_setopt($ch,CURLOPT_RETURNTRANSFER, TRUE);
           $result = curl_exec($ch);
           curl_close($ch);
	   
	   $response        = json_decode($result);
           //$operationResult = $response->retrieveIndividualAccountResponse->operationResult;
	   
           return $response;
        }
	
	//
        // AuthenticateUser API Login Function
        //
        // This function stores the user's account ID as a session variable. 
        // This is useful for calling up more data specific to this person.
        //
        function userLogin($username, $password, $sessionId)
        {
           $ch = curl_init();
           $url = 'https://api.neoncrm.com/neonws/services/api/common/authenticateUser';
           $params = 'userSessionId=' . $sessionId . '&username=' . $username . '&password=' . $password;
           curl_setopt($ch,CURLOPT_URL, $url);
           curl_setopt($ch,CURLOPT_POST, TRUE);
           curl_setopt($ch,CURLOPT_POSTFIELDS, $params);
           curl_setopt($ch,CURLOPT_RETURNTRANSFER, TRUE);
           $result = curl_exec($ch);
           curl_close($ch);
           $response = json_decode($result);
	   return $response;
        }

        //
        // Logout Function
        //
        // Once you have finished interacting with the NeonCRM API, 
        // use the logout() function to destroy your session.
        //
        function logout()
        {
           $ch = curl_init();
           $url = 'https://api.neoncrm.com/neonws/services/api/common/logout';
           $params = 'userSessionId=' . $_SESSION['session_id'];
           curl_setopt($ch,CURLOPT_URL, $url);
           curl_setopt($ch,CURLOPT_POST, TRUE);
           curl_setopt($ch,CURLOPT_POSTFIELDS, $params);
           curl_setopt($ch,CURLOPT_RETURNTRANSFER, TRUE);
           $result = curl_exec($ch);
           curl_close($ch);
           $response = json_decode($result);
           $message = $response->logoutResponse->responseMessage;
           return $message;
           session_unset();
           session_destroy();
      }
      
      function neon_auth( $username, $password ) {
		
		//DEBUG
		//echo "<p>neon_auth function called: Creating API Connection to Neon Server.</p>";
		
		$result = false;
		
		//
		// Establish the API connection to the Neon server
		//
		$loginResponse = $this->login($this->get_setting('api_key'), $this->get_setting('org_id'));
		
		//
		// The Neon Session ID
		//
		$sessionId = $loginResponse->loginResponse->userSessionId;
		
		//DEBUG
		//echo "<p>Returned from login... checking Users credentials with userLogin call. </p>";
		
		//
                // Is this user a valid, registered Neon user?
                // Login via the API connection using the supplied UserID and Password
		// and we'll see...
                //
                $NeonResponse = $this->userLogin($username, $password, $sessionId);

                //DEBUG
		//echo "<p>Returned from userLogin... checking results. </p>";
		
		//print_r($NeonResponse);
		
		$DGSMember = '';
                $Realname  = '';
                if ( $NeonResponse->authenticateUserResponse->operationResult == "SUCCESS" )
                {
                   //DEBUG
		   //echo "<p>Have a SUCCESS response</p>";
		
		   if ( $NeonResponse->authenticateUserResponse->isMember )
                   {
                      //DEBUG
		      //echo "<p>Login Successful - Current DGS Member</p>";
		      
		      $DGSMember = 'Yes';
                      $Realname  = $NeonResponse->authenticateUserResponse->name;
		      
		      //
		      // This users Neon Account ID
		      //
		      $accountId = $NeonResponse->authenticateUserResponse->accountId;
		      
		      //
		      // Get account information about this user
		      //
		      $userInfo = $this->RetrieveIndividualAccount($sessionId, $accountId);
		      
		      $firstName       = $userInfo->retrieveIndividualAccountResponse->individualAccount->primaryContact->firstName;
                      $lastName        = $userInfo->retrieveIndividualAccountResponse->individualAccount->primaryContact->lastName;
                      $email1          = $userInfo->retrieveIndividualAccountResponse->individualAccount->primaryContact->email1;
		      
		      $this->set_setting('first_name', $firstName);
		      $this->set_setting('last_name', $lastName);
		      $this->set_setting('email_address', $email1);
                   }
                }
		$result = $DGSMember;

                #
                # We're done with the API.. close the connection
                #
                $this->logout();

		return apply_filters($this->prefix . 'neon_auth', $result);
	}
}

if ( ! function_exists('str_true') ) {
	/**
	 * Evaluates natural language strings to boolean equivalent
	 *
	 * Used primarily for handling boolean text provided in shopp() tag options.
	 * All values defined as true will return true, anything else is false.
	 *
	 * Boolean values will be passed through.
	 *
	 * Replaces the 1.0-1.1 value_is_true()
	 *
	 * @author Jonathan Davis
	 * @since 1.2
	 *
	 * @param string $string The natural language value
	 * @param array $istrue A list strings that are true
	 * @return boolean The boolean value of the provided text
	 **/
	function str_true ( $string, $istrue = array('yes', 'y', 'true','1','on','open') ) {
		if (is_array($string)) return false;
		if (is_bool($string)) return $string;
		return in_array(strtolower($string),$istrue);
	}
}

$SimpleNeonLogin = SimpleNeonLogin::getInstance();